"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Id = void 0;
const web3_js_1 = require("@solana/web3.js");
const ids_json_1 = __importDefault(require("../ids.json"));
class Id {
    constructor(cluster, name, publicKey, serum3ProgramId, mangoProgramId, banks, stubOracles, mintInfos, serum3Markets, perpMarkets) {
        this.cluster = cluster;
        this.name = name;
        this.publicKey = publicKey;
        this.serum3ProgramId = serum3ProgramId;
        this.mangoProgramId = mangoProgramId;
        this.banks = banks;
        this.stubOracles = stubOracles;
        this.mintInfos = mintInfos;
        this.serum3Markets = serum3Markets;
        this.perpMarkets = perpMarkets;
    }
    getBanks() {
        return Array.from(this.banks.map((bank) => new web3_js_1.PublicKey(bank.publicKey)));
    }
    getStubOracles() {
        return Array.from(this.stubOracles.map((stubOracle) => new web3_js_1.PublicKey(stubOracle.publicKey)));
    }
    getMintInfos() {
        return Array.from(this.mintInfos.map((mintInfo) => new web3_js_1.PublicKey(mintInfo.publicKey)));
    }
    getSerum3Markets() {
        return Array.from(this.serum3Markets.map((serum3Market) => new web3_js_1.PublicKey(serum3Market.publicKey)));
    }
    getPerpMarkets() {
        return Array.from(this.perpMarkets.map((perpMarket) => new web3_js_1.PublicKey(perpMarket.publicKey)));
    }
    static fromIds(name) {
        const groupConfig = ids_json_1.default.groups.find((id) => id['name'] === name);
        return new Id(groupConfig.cluster, name, groupConfig.publicKey, groupConfig.serum3ProgramId, groupConfig.mangoProgramId, groupConfig['banks'], groupConfig['stubOracles'], groupConfig['mintInfos'], groupConfig['serum3Markets'], groupConfig['perpMarkets']);
    }
}
exports.Id = Id;
//# sourceMappingURL=ids.js.map