import { utf8 } from '@project-serum/anchor/dist/cjs/utils/bytes';
import { PublicKey } from '@solana/web3.js';
import { nativeI80F48ToUi } from '../utils';
import { I80F48, ZERO_I80F48 } from './I80F48';
export const QUOTE_DECIMALS = 6;
export class Bank {
    constructor(publicKey, name, group, mint, vault, oracle, oracleConfig, depositIndex, borrowIndex, indexedTotalDeposits, indexedTotalBorrows, indexedDeposits, indexedBorrows, indexLastUpdated, bankRateLastUpdated, avgUtilization, adjustmentFactor, util0, rate0, util1, rate1, maxRate, collectedFeesNative, loanOriginationFeeRate, loanFeeRate, maintAssetWeight, initAssetWeight, maintLiabWeight, initLiabWeight, liquidationFee, dust, flashLoanTokenAccountInitial, flashLoanApprovedAmount, tokenIndex, mintDecimals, bankNum) {
        this.publicKey = publicKey;
        this.group = group;
        this.mint = mint;
        this.vault = vault;
        this.oracle = oracle;
        this.indexLastUpdated = indexLastUpdated;
        this.bankRateLastUpdated = bankRateLastUpdated;
        this.tokenIndex = tokenIndex;
        this.mintDecimals = mintDecimals;
        this.bankNum = bankNum;
        this.name = utf8.decode(new Uint8Array(name)).split('\x00')[0];
        this.depositIndex = I80F48.from(depositIndex);
        this.borrowIndex = I80F48.from(borrowIndex);
        this.cachedIndexedTotalDeposits = I80F48.from(indexedTotalDeposits);
        this.cachedIndexedTotalBorrows = I80F48.from(indexedTotalBorrows);
        this.avgUtilization = I80F48.from(avgUtilization);
        this.adjustmentFactor = I80F48.from(adjustmentFactor);
        this.maxRate = I80F48.from(maxRate);
        this.util0 = I80F48.from(util0);
        this.rate0 = I80F48.from(rate0);
        this.util1 = I80F48.from(util1);
        this.rate1 = I80F48.from(rate1);
        this.maintAssetWeight = I80F48.from(maintAssetWeight);
        this.initAssetWeight = I80F48.from(initAssetWeight);
        this.maintLiabWeight = I80F48.from(maintLiabWeight);
        this.initLiabWeight = I80F48.from(initLiabWeight);
        this.liquidationFee = I80F48.from(liquidationFee);
        this.price = undefined;
    }
    static from(publicKey, obj) {
        return new Bank(publicKey, obj.name, obj.group, obj.mint, obj.vault, obj.oracle, obj.oracleConfig, obj.depositIndex, obj.borrowIndex, obj.cachedIndexedTotalDeposits, obj.cachedIndexedTotalBorrows, obj.indexedDeposits, obj.indexedBorrows, obj.indexLastUpdated, obj.bankRateLastUpdated, obj.avgUtilization, obj.adjustmentFactor, obj.util0, obj.rate0, obj.util1, obj.rate1, obj.maxRate, obj.collectedFeesNative, obj.loanOriginationFeeRate, obj.loanFeeRate, obj.maintAssetWeight, obj.initAssetWeight, obj.maintLiabWeight, obj.initLiabWeight, obj.liquidationFee, obj.dust, obj.flashLoanTokenAccountInitial, obj.flashLoanApprovedAmount, obj.tokenIndex, obj.mintDecimals, obj.bankNum);
    }
    toString() {
        return ('Bank ' +
            '\n token index - ' +
            this.tokenIndex +
            '\n token name - ' +
            this.name +
            '\n deposit index - ' +
            this.depositIndex.toNumber() +
            '\n borrow index - ' +
            this.borrowIndex.toNumber() +
            '\n cachedIndexedTotalDeposits - ' +
            this.cachedIndexedTotalDeposits.toNumber() +
            '\n cachedIndexedTotalBorrows - ' +
            this.cachedIndexedTotalBorrows.toNumber() +
            '\n indexLastUpdated - ' +
            new Date(this.indexLastUpdated.toNumber() * 1000) +
            '\n bankRateLastUpdated - ' +
            new Date(this.bankRateLastUpdated.toNumber() * 1000) +
            '\n avgUtilization - ' +
            this.avgUtilization.toNumber() +
            '\n adjustmentFactor - ' +
            this.adjustmentFactor.toNumber() +
            '\n maxRate - ' +
            this.maxRate.toNumber() +
            '\n util0 - ' +
            this.util0.toNumber() +
            '\n rate0 - ' +
            this.rate0.toNumber() +
            '\n util1 - ' +
            this.util1.toNumber() +
            '\n rate1 - ' +
            this.rate1.toNumber() +
            '\n maintAssetWeight - ' +
            this.maintAssetWeight.toNumber() +
            '\n initAssetWeight - ' +
            this.initAssetWeight.toNumber() +
            '\n maintLiabWeight - ' +
            this.maintLiabWeight.toNumber() +
            '\n initLiabWeight - ' +
            this.initLiabWeight.toNumber() +
            '\n liquidationFee - ' +
            this.liquidationFee.toNumber() +
            '\n uiDeposits() - ' +
            this.uiDeposits() +
            '\n uiBorrows() - ' +
            this.uiBorrows() +
            '\n getDepositRate() - ' +
            this.getDepositRate().toNumber() +
            '\n getBorrowRate() - ' +
            this.getBorrowRate().toNumber());
    }
    nativeDeposits() {
        return this.cachedIndexedTotalDeposits.mul(this.depositIndex);
    }
    nativeBorrows() {
        return this.cachedIndexedTotalBorrows.mul(this.borrowIndex);
    }
    uiDeposits() {
        return nativeI80F48ToUi(this.cachedIndexedTotalDeposits.mul(this.depositIndex), this.mintDecimals).toNumber();
    }
    uiBorrows() {
        return nativeI80F48ToUi(this.cachedIndexedTotalBorrows.mul(this.borrowIndex), this.mintDecimals).toNumber();
    }
    getBorrowRate() {
        const totalBorrows = this.nativeBorrows();
        const totalDeposits = this.nativeDeposits();
        if (totalDeposits.eq(ZERO_I80F48) && totalBorrows.eq(ZERO_I80F48)) {
            return ZERO_I80F48;
        }
        if (totalDeposits.lte(totalBorrows)) {
            return this.maxRate;
        }
        const utilization = totalBorrows.div(totalDeposits);
        if (utilization.gt(this.util1)) {
            const extraUtil = utilization.sub(this.util1);
            const slope = this.maxRate
                .sub(this.rate1)
                .div(I80F48.fromNumber(1).sub(this.util1));
            return this.rate1.add(slope.mul(extraUtil));
        }
        else if (utilization.gt(this.util0)) {
            const extraUtil = utilization.sub(this.util0);
            const slope = this.maxRate
                .sub(this.rate0)
                .div(I80F48.fromNumber(1).sub(this.util0));
            return this.rate0.add(slope.mul(extraUtil));
        }
        else {
            const slope = this.rate0.div(this.util0);
            return slope.mul(utilization);
        }
    }
    getDepositRate() {
        const borrowRate = this.getBorrowRate();
        const totalBorrows = this.nativeBorrows();
        const totalDeposits = this.nativeDeposits();
        if (totalDeposits.eq(ZERO_I80F48) && totalBorrows.eq(ZERO_I80F48)) {
            return ZERO_I80F48;
        }
        else if (totalDeposits.eq(ZERO_I80F48)) {
            return this.maxRate;
        }
        const utilization = totalBorrows.div(totalDeposits);
        return utilization.mul(borrowRate);
    }
}
export class MintInfo {
    constructor(publicKey, group, tokenIndex, mint, banks, vaults, oracle, registrationTime, groupInsuranceFund) {
        this.publicKey = publicKey;
        this.group = group;
        this.tokenIndex = tokenIndex;
        this.mint = mint;
        this.banks = banks;
        this.vaults = vaults;
        this.oracle = oracle;
        this.registrationTime = registrationTime;
        this.groupInsuranceFund = groupInsuranceFund;
    }
    static from(publicKey, obj) {
        return new MintInfo(publicKey, obj.group, obj.tokenIndex, obj.mint, obj.banks, obj.vaults, obj.oracle, obj.registrationTime, obj.groupInsuranceFund);
    }
    firstBank() {
        return this.banks[0];
    }
    firstVault() {
        return this.vaults[0];
    }
    toString() {
        const res = 'mint ' +
            this.mint.toBase58() +
            '\n oracle ' +
            this.oracle.toBase58() +
            '\n banks ' +
            this.banks
                .filter((pk) => pk.toBase58() !== PublicKey.default.toBase58())
                .toString() +
            '\n vaults ' +
            this.vaults
                .filter((pk) => pk.toBase58() !== PublicKey.default.toBase58())
                .toString();
        return res;
    }
}
//# sourceMappingURL=bank.js.map