import { BN } from '@project-serum/anchor';
import { utf8 } from '@project-serum/anchor/dist/cjs/utils/bytes';
import { QUOTE_DECIMALS } from './bank';
import { I80F48 } from './I80F48';
export class PerpMarket {
    constructor(publicKey, group, baseTokenIndex, quoteTokenIndex, perpMarketIndex, name, oracle, oracleConfig, bids, asks, eventQueue, quoteLotSize, baseLotSize, maintAssetWeight, initAssetWeight, maintLiabWeight, initLiabWeight, liquidationFee, makerFee, takerFee, minFunding, maxFundingI80F48Dto, impactQuantity, longFunding, shortFunding, fundingLastUpdated, openInterest, seqNum, feesAccrued, bump, baseTokenDecimals, registrationTime) {
        this.publicKey = publicKey;
        this.group = group;
        this.baseTokenIndex = baseTokenIndex;
        this.quoteTokenIndex = quoteTokenIndex;
        this.perpMarketIndex = perpMarketIndex;
        this.oracle = oracle;
        this.bids = bids;
        this.asks = asks;
        this.eventQueue = eventQueue;
        this.quoteLotSize = quoteLotSize;
        this.baseLotSize = baseLotSize;
        this.baseTokenDecimals = baseTokenDecimals;
        this.registrationTime = registrationTime;
        this.name = utf8.decode(new Uint8Array(name)).split('\x00')[0];
        this.maintAssetWeight = I80F48.from(maintAssetWeight);
        this.initAssetWeight = I80F48.from(initAssetWeight);
        this.maintLiabWeight = I80F48.from(maintLiabWeight);
        this.initLiabWeight = I80F48.from(initLiabWeight);
        this.liquidationFee = I80F48.from(liquidationFee);
        this.makerFee = I80F48.from(makerFee);
        this.takerFee = I80F48.from(takerFee);
        this.openInterest = openInterest.toNumber();
        this.seqNum = seqNum.toNumber();
        this.feesAccrued = I80F48.from(feesAccrued);
    }
    static from(publicKey, obj) {
        return new PerpMarket(publicKey, obj.group, obj.baseTokenIndex, obj.quoteTokenIndex, obj.perpMarketIndex, obj.name, obj.oracle, obj.oracleConfig, obj.bids, obj.asks, obj.eventQueue, obj.quoteLotSize, obj.baseLotSize, obj.maintAssetWeight, obj.initAssetWeight, obj.maintLiabWeight, obj.initLiabWeight, obj.liquidationFee, obj.makerFee, obj.takerFee, obj.minFunding, obj.maxFunding, obj.impactQuantity, obj.longFunding, obj.shortFunding, obj.fundingLastUpdated, obj.openInterest, obj.seqNum, obj.feesAccrued, obj.bump, obj.baseTokenDecimals, obj.registrationTime);
    }
    uiToNativePriceQuantity(price, quantity) {
        const baseUnit = Math.pow(10, this.baseTokenDecimals);
        const quoteUnit = Math.pow(10, QUOTE_DECIMALS);
        const nativePrice = new BN(price * quoteUnit)
            .mul(this.baseLotSize)
            .div(this.quoteLotSize.mul(new BN(baseUnit)));
        const nativeQuantity = new BN(quantity * baseUnit).div(this.baseLotSize);
        return [nativePrice, nativeQuantity];
    }
    uiQuoteToLots(uiQuote) {
        const quoteUnit = Math.pow(10, QUOTE_DECIMALS);
        return new BN(uiQuote * quoteUnit).div(this.quoteLotSize);
    }
    toString() {
        return ('PerpMarket ' +
            '\n perpMarketIndex -' +
            this.perpMarketIndex +
            '\n maintAssetWeight -' +
            this.maintAssetWeight.toNumber() +
            '\n initAssetWeight -' +
            this.initAssetWeight.toNumber() +
            '\n maintLiabWeight -' +
            this.maintLiabWeight.toNumber() +
            '\n initLiabWeight -' +
            this.initLiabWeight.toNumber() +
            '\n liquidationFee -' +
            this.liquidationFee.toNumber() +
            '\n makerFee -' +
            this.makerFee.toNumber() +
            '\n takerFee -' +
            this.takerFee.toNumber());
    }
}
export class Side {
}
Side.bid = { bid: {} };
Side.ask = { ask: {} };
export class OrderType {
}
OrderType.limit = { limit: {} };
OrderType.immediateOrCancel = { immediateorcancel: {} };
OrderType.postOnly = { postonly: {} };
OrderType.market = { market: {} };
OrderType.postOnlySlide = { postonlyslide: {} };
//# sourceMappingURL=perp.js.map