/// <reference types="bn.js" />
import { BN, Program, Provider } from '@project-serum/anchor';
import { Order } from '@project-serum/serum/lib/market';
import { Cluster, PublicKey, TransactionInstruction, TransactionSignature } from '@solana/web3.js';
import { Bank, MintInfo } from './accounts/bank';
import { Group } from './accounts/group';
import { MangoAccount, MangoAccountData } from './accounts/mangoAccount';
import { StubOracle } from './accounts/oracle';
import { OrderType, PerpMarket, Side } from './accounts/perp';
import { Serum3Market, Serum3OrderType, Serum3SelfTradeBehavior, Serum3Side } from './accounts/serum3';
import { MangoV4 } from './mango_v4';
declare enum AccountRetriever {
    Scanning = 0,
    Fixed = 1
}
export declare class MangoClient {
    program: Program<MangoV4>;
    programId: PublicKey;
    cluster: Cluster;
    groupName?: string;
    constructor(program: Program<MangoV4>, programId: PublicKey, cluster: Cluster, groupName?: string);
    groupCreate(groupNum: number, testing: boolean, version: number, insuranceMintPk: PublicKey): Promise<TransactionSignature>;
    groupEdit(group: Group, newAdmin: PublicKey, newFastListingAdmin: PublicKey): Promise<TransactionSignature>;
    groupClose(group: Group): Promise<TransactionSignature>;
    getGroup(groupPk: PublicKey): Promise<Group>;
    getGroupForCreator(creatorPk: PublicKey, groupNum?: number): Promise<Group>;
    tokenRegister(group: Group, mintPk: PublicKey, oraclePk: PublicKey, oracleConfFilter: number, tokenIndex: number, name: string, adjustmentFactor: number, util0: number, rate0: number, util1: number, rate1: number, maxRate: number, loanFeeRate: number, loanOriginationFeeRate: number, maintAssetWeight: number, initAssetWeight: number, maintLiabWeight: number, initLiabWeight: number, liquidationFee: number): Promise<TransactionSignature>;
    tokenRegisterTrustless(group: Group, mintPk: PublicKey, oraclePk: PublicKey, tokenIndex: number, name: string): Promise<TransactionSignature>;
    tokenEdit(group: Group, tokenName: string, oracle: PublicKey, oracleConfFilter: number, groupInsuranceFund: boolean | undefined, adjustmentFactor: number, util0: number, rate0: number, util1: number, rate1: number, maxRate: number, loanFeeRate: number, loanOriginationFeeRate: number, maintAssetWeight: number, initAssetWeight: number, maintLiabWeight: number, initLiabWeight: number, liquidationFee: number): Promise<TransactionSignature>;
    tokenDeregister(group: Group, tokenName: string): Promise<TransactionSignature>;
    getBanksForGroup(group: Group): Promise<Bank[]>;
    getMintInfosForGroup(group: Group): Promise<MintInfo[]>;
    getMintInfoForTokenIndex(group: Group, tokenIndex: number): Promise<MintInfo[]>;
    stubOracleCreate(group: Group, mintPk: PublicKey, price: number): Promise<TransactionSignature>;
    stubOracleClose(group: Group, oracle: PublicKey): Promise<TransactionSignature>;
    stubOracleSet(group: Group, oraclePk: PublicKey, price: number): Promise<TransactionSignature>;
    getStubOracle(group: Group, mintPk?: PublicKey): Promise<StubOracle[]>;
    getOrCreateMangoAccount(group: Group, ownerPk: PublicKey, accountNumber?: number, name?: string): Promise<MangoAccount>;
    createMangoAccount(group: Group, accountNumber?: number, name?: string): Promise<TransactionSignature>;
    expandMangoAccount(group: Group, account: MangoAccount, tokenCount: number, serum3Count: number, perpCount: number, perpOoCount: number): Promise<TransactionSignature>;
    editMangoAccount(group: Group, mangoAccount: MangoAccount, name?: string, delegate?: PublicKey): Promise<TransactionSignature>;
    getMangoAccount(mangoAccount: MangoAccount): Promise<MangoAccount>;
    getMangoAccountsForOwner(group: Group, ownerPk: PublicKey): Promise<MangoAccount[]>;
    closeMangoAccount(group: Group, mangoAccount: MangoAccount): Promise<TransactionSignature>;
    computeAccountData(group: Group, mangoAccount: MangoAccount): Promise<MangoAccountData>;
    tokenDeposit(group: Group, mangoAccount: MangoAccount, tokenName: string, amount: number): Promise<TransactionSignature>;
    tokenWithdraw(group: Group, mangoAccount: MangoAccount, tokenName: string, amount: number, allowBorrow: boolean): Promise<TransactionSignature>;
    tokenWithdrawNative(group: Group, mangoAccount: MangoAccount, tokenName: string, nativeAmount: number, allowBorrow: boolean): Promise<TransactionSignature>;
    serum3RegisterMarket(group: Group, serum3MarketExternalPk: PublicKey, baseBank: Bank, quoteBank: Bank, marketIndex: number, name: string): Promise<TransactionSignature>;
    serum3deregisterMarket(group: Group, serum3MarketName: string): Promise<TransactionSignature>;
    serum3GetMarkets(group: Group, baseTokenIndex?: number, quoteTokenIndex?: number): Promise<Serum3Market[]>;
    serum3CreateOpenOrders(group: Group, mangoAccount: MangoAccount, marketName: string): Promise<TransactionSignature>;
    serum3CloseOpenOrders(group: Group, mangoAccount: MangoAccount, serum3MarketName: string): Promise<TransactionSignature>;
    serum3PlaceOrder(group: Group, mangoAccount: MangoAccount, serum3MarketName: string, side: Serum3Side, price: number, size: number, selfTradeBehavior: Serum3SelfTradeBehavior, orderType: Serum3OrderType, clientOrderId: number, limit: number): Promise<string>;
    serum3CancelAllorders(group: Group, mangoAccount: MangoAccount, serum3MarketName: string, limit: number): Promise<string>;
    serum3SettleFunds(group: Group, mangoAccount: MangoAccount, serum3MarketName: string): Promise<TransactionSignature>;
    serum3CancelOrder(group: Group, mangoAccount: MangoAccount, serum3MarketName: string, side: Serum3Side, orderId: BN): Promise<TransactionSignature>;
    getSerum3Orders(group: Group, serum3MarketName: string): Promise<Order[]>;
    perpCreateMarket(group: Group, oraclePk: PublicKey, perpMarketIndex: number, name: string, oracleConfFilter: number, baseTokenIndex: number, baseTokenDecimals: number, quoteTokenIndex: number, quoteLotSize: number, baseLotSize: number, maintAssetWeight: number, initAssetWeight: number, maintLiabWeight: number, initLiabWeight: number, liquidationFee: number, makerFee: number, takerFee: number, minFunding: number, maxFunding: number, impactQuantity: number): Promise<TransactionSignature>;
    perpEditMarket(group: Group, perpMarketName: string, oracle: PublicKey, oracleConfFilter: number, baseTokenIndex: number, baseTokenDecimals: number, maintAssetWeight: number, initAssetWeight: number, maintLiabWeight: number, initLiabWeight: number, liquidationFee: number, makerFee: number, takerFee: number, minFunding: number, maxFunding: number, impactQuantity: number): Promise<TransactionSignature>;
    perpCloseMarket(group: Group, perpMarketName: string): Promise<TransactionSignature>;
    perpGetMarkets(group: Group, baseTokenIndex?: number): Promise<PerpMarket[]>;
    perpPlaceOrder(group: Group, mangoAccount: MangoAccount, perpMarketName: string, side: Side, price: number, quantity: number, maxQuoteQuantity: number, clientOrderId: number, orderType: OrderType, expiryTimestamp: number, limit: number): Promise<void>;
    marginTrade({ group, mangoAccount, inputToken, amountIn, outputToken, userDefinedInstructions, }: {
        group: Group;
        mangoAccount: MangoAccount;
        inputToken: string;
        amountIn: number;
        outputToken: string;
        userDefinedInstructions: TransactionInstruction[];
    }): Promise<TransactionSignature>;
    updateIndexAndRate(group: Group, tokenName: string): Promise<void>;
    liqTokenWithToken(group: Group, liqor: MangoAccount, liqee: MangoAccount, assetTokenName: string, liabTokenName: string, maxLiabTransfer: number): Promise<void>;
    static connect(provider: Provider, cluster: Cluster, programId: PublicKey): MangoClient;
    static connectForGroupName(provider: Provider, groupName: string): MangoClient;
    buildHealthRemainingAccounts(retriever: AccountRetriever, group: Group, mangoAccounts: MangoAccount[], banks?: Bank[] /** TODO for serum3PlaceOrder we are just ingoring this atm */): PublicKey[];
    buildFixedAccountRetrieverHealthAccounts(group: Group, mangoAccount: MangoAccount, banks?: Bank[] /** TODO for serum3PlaceOrder we are just ingoring this atm */): PublicKey[];
    buildScanningAccountRetrieverHealthAccounts(group: Group, mangoAccounts: MangoAccount[], banks?: Bank[] /** TODO for serum3PlaceOrder we are just ingoring this atm */): PublicKey[];
}
export {};
